"""Example aggregate module."""

from .aggregate import Example
from .command_handlers import ExampleCommandHandler
from .commands import ActivateExample, ArchiveExample, CreateExample
from .event_handlers import ExampleEventHandler
from .events import ExampleActivated, ExampleArchived, ExampleCreated
from .repository import ExampleRepository
from .value_objects import Address

__all__ = [
    "Example",
    "ExampleRepository",
    "CreateExample",
    "ActivateExample",
    "ArchiveExample",
    "ExampleCreated",
    "ExampleActivated",
    "ExampleArchived",
    "ExampleCommandHandler",
    "ExampleEventHandler",
    "Address",
]
